use data::tables;

use crate::ServerState;

pub async fn avatar(
    args: &[&str],
    state: &ServerState,
) -> Result<String, Box<dyn std::error::Error>> {
    const USAGE: &str = "Usage: player avatar [player_uid] [avatar_id]";

    if args.len() != 2 {
        return Ok(USAGE.to_string());
    }

    let uid = args[0].parse::<u32>()?;
    let avatar_id = args[1].parse::<u32>()?;

    let Some(player_lock) = state.player_mgr.get_player(uid).await else {
        return Ok(String::from("player not found"));
    };

    if !tables::avatar_base_template_tb::iter().any(|tmpl| tmpl.id == avatar_id) {
        return Ok(format!("section with id {avatar_id} doesn't exist"));
    }

    let should_save = {
        let mut player = player_lock.lock().await;
        player.basic_data_model.frontend_avatar_id = avatar_id as i32;
        player.current_session_id().is_none()
    };

    if should_save {
        state.player_mgr.save_and_remove(uid).await;
    }

    Ok(format!(
        "changed frontend_avatar_id, you have to re-enter main city now"
    ))
}
