use super::ResourceItem;
use proto::ItemModelBin;

pub struct ItemModel {
    pub resources: Vec<ResourceItem>,
}

impl ItemModel {
    pub fn from_bin(bin: ItemModelBin) -> Self {
        Self {
            resources: bin
                .resource_list
                .into_iter()
                .map(ResourceItem::from_bin)
                .collect(),
        }
    }

    pub fn to_bin(&self) -> ItemModelBin {
        ItemModelBin {
            resource_list: self.resources.iter().map(ResourceItem::to_bin).collect(),
        }
    }

    pub fn add_resource(&mut self, template_id: u32, num: u32) {
        if let Some(item) = self
            .resources
            .iter_mut()
            .find(|item| item.template_id == template_id)
        {
            item.num += num as i32;
        } else {
            self.resources.push(ResourceItem {
                template_id,
                num: num as i32,
            });
        }
    }
}

impl Default for ItemModel {
    fn default() -> Self {
        Self {
            resources: Vec::new(),
        }
    }
}
